/*
 * \brief  Tpm rpc client
 * \author Jean-Adrien Domage <jean-adrien.domage@gapfruit.com>
 * \date   2021-06-24
 */

/*
 * Copyright (C) 2021 gapfruit AG
 */

#ifndef _INCLUDE__TPM__CLIENT_H_
#define _INCLUDE__TPM__CLIENT_H_


#include <base/mutex.h>
#include <base/rpc_client.h>
#include <base/attached_dataspace.h>
#include <util/string.h>

#include <tpm_session/tpm_session.h>

namespace Tpm {

	using namespace Genode;

	class Session_client;

}

class Tpm::Session_client: public Rpc_client<Session>
{
private:

	Attached_dataspace _tx_buffer;
	Attached_dataspace _rx_buffer;

public:

	Session_client(Region_map &rm, Capability<Session> cap)
	:
		Rpc_client<Session> { cap },
		_tx_buffer          { rm, call<Rpc_tx_dataspace>() },
		_rx_buffer          { rm, call<Rpc_rx_dataspace>() }
	{ }

	/* transfer() is NOT thread-safe */
	void invoke(uint8_t const *tx_buffer, uint8_t *rx_buffer, size_t buffers_size) override {
		/* throw an exception if there are no buffer at all to operate on */
		/* throw an exception if we don't have enough memory to operate */
		if (buffers_size == 0
		    || (!tx_buffer || !rx_buffer)
		    || buffers_size > _tx_buffer.size()
		    || buffers_size > _rx_buffer.size()) {
			throw Tpm::Communication_error();
		}

		memcpy(_tx_buffer.local_addr<uint8_t>(), tx_buffer, buffers_size);
		try {
			call<Rpc_invoke>(buffers_size);
		}
		catch (...) {
			throw Tpm::Communication_error();
		}
		memcpy(rx_buffer, _rx_buffer.local_addr<uint8_t>(), buffers_size);
	}

};

#endif // _INCLUDE__TPM__CLIENT_H_
